@@ -296,10 +296,19 @@ def _parseConfigfileCache(cache_dict, dirpath):
                                for tier_dict in cache_dict['tiers']]
     
         elif _class is Caches.Memcache.Cache:
+            if 'key prefix' in cache_dict:
+                kwargs['key_prefix'] = cache_dict['key prefix']
+        
             add_kwargs('servers', 'lifespan', 'revision')
+
+        elif _class is Caches.Redis.Cache:
+            if 'key prefix' in cache_dict:
+                kwargs['key_prefix'] = cache_dict['key prefix']
+
+            add_kwargs('host', 'port', 'db')
     
         elif _class is Caches.S3.Cache:
-            add_kwargs('bucket', 'access', 'secret')
+            add_kwargs('bucket', 'access', 'secret', 'use_locks', 'path', 'reduced_redundancy')
     
         else:
             raise Exception('Unknown cache: %s' % cache_dict['name'])
@@ -361,6 +370,9 @@ def _parseConfigfileLayer(layer_dict, config, dirpath):
     if 'redirects' in layer_dict:
         layer_kwargs['redirects'] = dict(layer_dict['redirects'])
     
+    if 'tile height' in layer_dict:
+        layer_kwargs['tile_height'] = int(layer_dict['tile height'])
+    
     if 'preview' in layer_dict:
         preview_dict = layer_dict['preview']
         
@@ -417,44 +429,8 @@ def _parseConfigfileLayer(layer_dict, config, dirpath):
 
     if 'name' in provider_dict:
         _class = Providers.getProviderByName(provider_dict['name'])
-        provider_kwargs = {}
+        provider_kwargs = _class.prepareKeywordArgs(provider_dict)
         
-        if _class is Providers.Mapnik:
-            provider_kwargs['mapfile'] = provider_dict['mapfile']
-            provider_kwargs['fonts'] = provider_dict.get('fonts', None)
-        
-        elif _class is Providers.Proxy:
-            if 'url' in provider_dict:
-                provider_kwargs['url'] = provider_dict['url']
-            if 'provider' in provider_dict:
-                provider_kwargs['provider_name'] = provider_dict['provider']
-        
-        elif _class is Providers.UrlTemplate:
-            provider_kwargs['template'] = provider_dict['template']
-            if 'referer' in provider_dict:
-                provider_kwargs['referer'] = provider_dict['referer']
-        
-        elif _class is Providers.Vector.Provider:
-            provider_kwargs['driver'] = provider_dict['driver']
-            provider_kwargs['parameters'] = provider_dict['parameters']
-            provider_kwargs['properties'] = provider_dict.get('properties', None)
-            provider_kwargs['projected'] = bool(provider_dict.get('projected', False))
-            provider_kwargs['verbose'] = bool(provider_dict.get('verbose', False))
-            provider_kwargs['precision'] = int(provider_dict.get('precision', 6))
-            
-            if 'spacing' in provider_dict:
-                provider_kwargs['spacing'] = float(provider_dict.get('spacing', 0.0))
-            else:
-                provider_kwargs['spacing'] = None
-            
-            if provider_dict.get('clipped', None) == 'padded':
-                provider_kwargs['clipped'] = 'padded'
-            else:
-                provider_kwargs['clipped'] = bool(provider_dict.get('clipped', True))
-        
-        elif _class is Providers.MBTiles.Provider:
-            provider_kwargs['tileset'] = provider_dict['tileset']
-            
     elif 'class' in provider_dict:
         _class = loadClassPath(provider_dict['class'])
         provider_kwargs = provider_dict.get('kwargs', {})
