@@ -23,6 +23,8 @@ configuration file as a dictionary:
           "bounds": { ... },
           "allowed origin": ...,
           "maximum cache age": ...,
+          "redirects": ...,
+          "tile height": ...,
           "jpeg options": ...,
           "png options": ...
         }
@@ -64,6 +66,9 @@ configuration file as a dictionary:
   If a request is made for a tile with an extension in the dictionary keys,
   a response can be generated that redirects the client to the same tile
   with another extension.
+- "tile height" gives the height of the image tile in pixels. You almost always
+  want to leave this at the default value of 256, but you can use a value of 512
+  to create double-size, double-resolution tiles for high-density phone screens.
 - "jpeg options" is an optional dictionary of JPEG creation options, passed
   through to PIL: http://www.pythonware.com/library/pil/handbook/format-jpeg.htm.
 - "png options" is an optional dictionary of PNG creation options, passed
@@ -129,6 +134,7 @@ The preview can be accessed through a URL like /<layer name>/preview.html:
 """
 
 import logging
+from wsgiref.headers import Headers
 from StringIO import StringIO
 from urlparse import urljoin
 from time import time
@@ -298,8 +304,13 @@ class Layer:
 
           preview_ext:
             Tile name extension for slippy map layer preview, default "png".
+
+          tile_height:
+            Height of tile in pixels, as a single integer. Tiles are generally
+            assumed to be square, and Layer.render() will respond with an error
+            if the rendered image is not this height.
     """
-    def __init__(self, config, projection, metatile, stale_lock_timeout=15, cache_lifespan=None, write_cache=True, allowed_origin=None, max_cache_age=None, redirects=None, preview_lat=37.80, preview_lon=-122.26, preview_zoom=10, preview_ext='png', bounds=None):
+    def __init__(self, config, projection, metatile, stale_lock_timeout=15, cache_lifespan=None, write_cache=True, allowed_origin=None, max_cache_age=None, redirects=None, preview_lat=37.80, preview_lon=-122.26, preview_zoom=10, preview_ext='png', bounds=None, tile_height=256):
         self.provider = None
         self.config = config
         self.projection = projection
@@ -318,6 +329,7 @@ class Layer:
         self.preview_ext = preview_ext
         
         self.bounds = bounds
+        self.dim = tile_height
         
         self.bitmap_palette = None
         self.jpeg_options = {}
@@ -335,6 +347,112 @@ class Layer:
 
         return None
 
+    def getTileResponse(self, coord, extension, ignore_cached=False):
+        """ Get status code, headers, and a tile binary for a given request layer tile.
+        
+            Arguments:
+            - coord: one ModestMaps.Core.Coordinate corresponding to a single tile.
+            - extension: filename extension to choose response type, e.g. "png" or "jpg".
+            - ignore_cached: always re-render the tile, whether it's in the cache or not.
+        
+            This is the main entry point, after site configuration has been loaded
+            and individual tiles need to be rendered.
+        """
+        start_time = time()
+        
+        mimetype, format = self.getTypeByExtension(extension)
+
+        # default response values
+        status_code = 200
+        headers = Headers([('Content-Type', mimetype)])
+        body = None
+
+        cache = self.config.cache
+
+        if not ignore_cached:
+            # Start by checking for a tile in the cache.
+            try:
+                body = cache.read(self, coord, format)
+            except TheTileLeftANote, e:
+                headers = e.headers
+                status_code = e.status_code
+                body = e.content
+
+                if e.emit_content_type:
+                    headers.setdefault('Content-Type', mimetype)
+
+            tile_from = 'cache'
+
+        else:
+            # Then look in the bag of recent tiles.
+            body = _getRecentTile(self, coord, format)
+            tile_from = 'recent tiles'
+        
+        # If no tile was found, dig deeper
+        if body is None:
+            try:
+                lockCoord = None
+
+                if self.write_cache:
+                    # this is the coordinate that actually gets locked.
+                    lockCoord = self.metatile.firstCoord(coord)
+                    
+                    # We may need to write a new tile, so acquire a lock.
+                    cache.lock(self, lockCoord, format)
+                
+                if not ignore_cached:
+                    # There's a chance that some other process has
+                    # written the tile while the lock was being acquired.
+                    body = cache.read(self, coord, format)
+                    tile_from = 'cache after all'
+        
+                if body is None:
+                    # No one else wrote the tile, do it here.
+                    buff = StringIO()
+
+                    try:
+                        tile = self.render(coord, format)
+                        save = True
+                    except NoTileLeftBehind, e:
+                        tile = e.tile
+                        save = False
+
+                    if not self.write_cache:
+                        save = False
+                    
+                    if format.lower() == 'jpeg':
+                        save_kwargs = self.jpeg_options
+                    elif format.lower() == 'png':
+                        save_kwargs = self.png_options
+                    else:
+                        save_kwargs = {}
+                    
+                    tile.save(buff, format, **save_kwargs)
+                    body = buff.getvalue()
+                    
+                    if save:
+                        cache.save(body, self, coord, format)
+
+                    tile_from = 'layer.render()'
+
+            except TheTileLeftANote, e:
+                headers = e.headers
+                status_code = e.status_code
+                body = e.content
+                
+                if e.emit_content_type:
+                    headers.setdefault('Content-Type', mimetype)
+
+            finally:
+                if lockCoord:
+                    # Always clean up a lock when it's no longer being used.
+                    cache.unlock(self, lockCoord, format)
+        
+        _addRecentTile(self, coord, format, body)
+        logging.info('TileStache.Core.Layer.getTileResponse() %s/%d/%d/%d.%s via %s in %.3f', self.name(), coord.zoom, coord.column, coord.row, extension, tile_from, time() - start_time)
+        
+        return status_code, headers, body
+
     def doMetatile(self):
         """ Return True if we have a real metatile and the provider is OK with it.
         """
@@ -345,18 +463,27 @@ class Layer:
         
             Perform metatile slicing here as well, if required, writing the
             full set of rendered tiles to cache as we go.
+
+            Note that metatiling and pass-through mode of a Provider
+            are mutually exclusive options
         """
         if self.bounds and self.bounds.excludes(coord):
-            raise NoTileLeftBehind(Image.new('RGB', (256, 256), (0x99, 0x99, 0x99)))
+            raise NoTileLeftBehind(Image.new('RGB', (self.dim, self.dim), (0x99, 0x99, 0x99)))
         
         srs = self.projection.srs
         xmin, ymin, xmax, ymax = self.envelope(coord)
-        width, height = 256, 256
+        width, height = self.dim, self.dim
         
         provider = self.provider
         metatile = self.metatile
+        pass_through = provider.pass_through if hasattr(provider, 'pass_through') else False
+
         
         if self.doMetatile():
+
+            if pass_through:
+                raise KnownUnknown('Your provider is configured for metatiling and pass_through mode. That does not work')
+
             # adjust render size and coverage for metatile
             xmin, ymin, xmax, ymax = self.metaEnvelope(coord)
             width, height = self.metaSize(coord)
@@ -369,7 +496,7 @@ class Layer:
         
         elif hasattr(provider, 'renderTile'):
             # draw a single tile
-            width, height = 256, 256
+            width, height = self.dim, self.dim
             tile = provider.renderTile(width, height, srs, coord)
 
         else:
@@ -378,12 +505,15 @@ class Layer:
         if not hasattr(tile, 'save'):
             raise KnownUnknown('Return value of provider.renderArea() must act like an image; e.g. have a "save" method.')
 
-        if hasattr(tile, 'size') and tile.size != (width, height):
-            raise KnownUnknown('Your provider returned the wrong image size: %s.' % repr(tile.size))
+        if hasattr(tile, 'size') and tile.size[1] != height:
+            raise KnownUnknown('Your provider returned the wrong image size: %s instead of %d pixels tall.' % (repr(tile.size), self.dim))
         
         if self.bitmap_palette:
             # this is where we apply the palette if there is one
 
+            if pass_through:
+                raise KnownUnknown('Cannot apply palette in pass_through mode')
+
             if format.lower() == 'png':
                 t_index = self.png_options.get('transparency', None)
                 tile = apply_palette(tile, self.bitmap_palette, t_index)
@@ -394,7 +524,7 @@ class Layer:
             
             for (other, x, y) in subtiles:
                 buff = StringIO()
-                bbox = (x, y, x + 256, y + 256)
+                bbox = (x, y, x + self.dim, y + self.dim)
                 subtile = surtile.crop(bbox)
                 subtile.save(buff, format)
                 body = buff.getvalue()
@@ -422,7 +552,7 @@ class Layer:
         """ Projected rendering envelope (xmin, ymin, xmax, ymax) for a metatile.
         """
         # size of buffer expressed as fraction of tile size
-        buffer = float(self.metatile.buffer) / 256
+        buffer = float(self.metatile.buffer) / self.dim
         
         # full set of metatile coordinates
         coords = self.metatile.allCoords(coord)
@@ -442,11 +572,11 @@ class Layer:
         """ Pixel width and height of full rendered image for a metatile.
         """
         # size of buffer expressed as fraction of tile size
-        buffer = float(self.metatile.buffer) / 256
+        buffer = float(self.metatile.buffer) / self.dim
         
         # new master image render size
-        width = int(256 * (buffer * 2 + self.metatile.columns))
-        height = int(256 * (buffer * 2 + self.metatile.rows))
+        width = int(self.dim * (buffer * 2 + self.metatile.columns))
+        height = int(self.dim * (buffer * 2 + self.metatile.rows))
         
         return width, height
 
@@ -461,8 +591,8 @@ class Layer:
             r = other.row - coords[0].row
             c = other.column - coords[0].column
             
-            x = c * 256 + self.metatile.buffer
-            y = r * 256 + self.metatile.buffer
+            x = c * self.dim + self.metatile.buffer
+            y = r * self.dim + self.metatile.buffer
             
             subtiles.append((other, x, y))
 
@@ -543,15 +673,22 @@ class NoTileLeftBehind(Exception):
         self.tile = tile
         Exception.__init__(self, tile)
 
-class TheTileIsInAnotherCastle(Exception):
-    """ Ask a client to look someplace else for a tile.
-    
-        This exception can be thrown in a provider to signal
-        to HTTP clients that a tile should be asked-for elsewhere.
+class TheTileLeftANote(Exception):
+    """ A tile exists, but it shouldn't be returned to the client. Headers
+        and/or a status code are provided in its stead.
+
+        This exception can be thrown in a provider or a cache to signal to
+        upstream servers where a tile can be found or to clients that a tile
+        is empty (or solid).
     """
-    def __init__(self, path_info):
-        self.path_info = path_info
-        Exception.__init__(self, path_info)
+    def __init__(self, headers=None, status_code=200, content='', emit_content_type=True):
+        self.headers = headers or Headers([])
+        self.status_code = status_code
+        self.content = content
+        self.emit_content_type = bool(emit_content_type)
+
+        Exception.__init__(self, self.headers, self.status_code,
+                           self.content, self.emit_content_type)
 
 def _preview(layer):
     """ Get an HTML response for a given named layer.
@@ -575,6 +712,11 @@ def _preview(layer):
             margin: 0;
             padding: 0;
         }
+
+        #map img {
+            width: 256px;
+            height: 256px;
+        }
     </style>
 </head>
 <body>
