@@ -19,6 +19,12 @@ as in this example stack that simply echoes another layer:
 
     {"src": "layer-name"}
 
+Layers can be limited to appear at certain zoom levels, given either as a range
+or as a single number:
+
+    {"src": "layer-name", "zoom": "12"}
+    {"src": "layer-name", "zoom": "12-18"}
+
 Layers can also be used as masks, as in this example that uses one layer
 to mask another layer:
 
@@ -142,6 +148,7 @@ This complete example can be found in the included examples directory.
 """
 
 import sys
+import re
 
 from urllib import urlopen
 from urlparse import urljoin
@@ -227,21 +234,22 @@ class Composite(Provider):
     """
     pass
 
-def build_stack(object):
+def build_stack(obj):
     """ Build up a data structure of Stack and Layer objects from lists of dictionaries.
     
         Normally, this is applied to the "stack" parameter to Composite.Provider.
     """
-    if type(object) is list:
-        layers = map(build_stack, object)
+    if type(obj) is list:
+        layers = map(build_stack, obj)
         return Stack(layers)
     
-    elif type(object) is dict:
-        keys = ('src', 'layername'), ('color', 'colorname'), \
-               ('mask', 'maskname'), ('opacity', 'opacity'), \
-               ('mode', 'blendmode'), ('adjustments', 'adjustments')
+    elif type(obj) is dict:
+        keys = (('src', 'layername'), ('color', 'colorname'),
+                ('mask', 'maskname'), ('opacity', 'opacity'),
+                ('mode', 'blendmode'), ('adjustments', 'adjustments'),
+                ('zoom', 'zoom'))
 
-        args = [(arg, object[key]) for (key, arg) in keys if key in object]
+        args = [(arg, obj[key]) for (key, arg) in keys if key in obj]
         
         return Layer(**dict(args))
 
@@ -254,9 +262,10 @@ class Layer:
         Can include a reference to another layer for the source image, a second
         reference to another layer for the mask, and a color name for the fill.
     """
-    def __init__(self, layername=None, colorname=None, maskname=None, opacity=1.0, blendmode=None, adjustments=None):
+    def __init__(self, layername=None, colorname=None, maskname=None, opacity=1.0,
+                       blendmode=None, adjustments=None, zoom=""):
         """ A new image layer.
-        
+
             Arguments:
             
               layername:
@@ -274,7 +283,25 @@ class Layer:
         self.opacity = opacity
         self.blendmode = blendmode
         self.adjustments = adjustments
+        
+        zooms = re.search("^(\d+)-(\d+)$|^(\d+)$", zoom) if zoom else None
+        
+        if zooms:
+            min_zoom, max_zoom, at_zoom = zooms.groups()
+            
+            if min_zoom is not None and max_zoom is not None:
+                self.min_zoom, self.max_zoom = int(min_zoom), int(max_zoom)
+            elif at_zoom is not None:
+                self.min_zoom, self.max_zoom = int(at_zoom), int(at_zoom)
+        
+        else:
+            self.min_zoom, self.max_zoom = 0, float('inf')
 
+    def in_zoom(self, zoom):
+        """ Return true if the requested zoom level is valid for this layer.
+        """
+        return self.min_zoom <= zoom and zoom <= self.max_zoom
+    
     def render(self, config, input_rgba, coord):
         """ Render this image layer.
 
@@ -340,6 +367,9 @@ class Layer:
 
         return output_rgba
 
+    def __str__(self):
+        return self.layername
+
 class Stack:
     """ A stack of image layers.
     """
@@ -353,6 +383,11 @@ class Stack:
         """
         self.layers = layers
 
+    def in_zoom(self, level):
+        """
+        """
+        return True
+    
     def render(self, config, input_rgba, coord):
         """ Render this image stack.
 
@@ -364,7 +399,9 @@ class Stack:
         
         for layer in self.layers:
             try:
-                stack_rgba = layer.render(config, stack_rgba, coord)
+                if layer.in_zoom(coord.zoom):
+                    stack_rgba = layer.render(config, stack_rgba, coord)
+
             except IOError:
                 # Be permissive of I/O errors getting sub-layers, for example if a
                 # proxy layer referenced here doesn't have an image for a zoom level.
@@ -621,7 +658,7 @@ def blend_images(bottom_rgba, top_rgb, mask_chan, opacity, blendmode):
         if blendmode in blend_functions:
             for c in (0, 1, 2):
                 blend_function = blend_functions[blendmode]
-                blend_function(output_rgba[c], bottom_rgba[c], top_rgb[c])
+                output_rgba[c] = blend_function(bottom_rgba[c], top_rgb[c])
         
         else:
             raise KnownUnknown('Unrecognized blend mode: "%s"' % blendmode)
@@ -653,41 +690,44 @@ def blend_images(bottom_rgba, top_rgb, mask_chan, opacity, blendmode):
             output_rgba[c][~nz] = 0
     
     # output mask is the screen of the existing and overlaid alphas
-    blend_channels_screen(output_rgba[3], bottom_rgba[3], mask_chan)
+    output_rgba[3] = blend_channels_screen(bottom_rgba[3], mask_chan)
 
     return output_rgba
 
-def blend_channels_screen(output_chan, bottom_chan, top_chan):
-    """ Modify output channel in-place with the combination of bottom and top channels.
+def blend_channels_screen(bottom_chan, top_chan):
+    """ Return combination of bottom and top channels.
     
         Math from http://illusions.hu/effectwiki/doku.php?id=screen_blending
     """
-    output_chan[:,:] = 1 - (1 - bottom_chan[:,:]) * (1 - top_chan[:,:])
+    return 1 - (1 - bottom_chan[:,:]) * (1 - top_chan[:,:])
 
-def blend_channels_multiply(output_chan, bottom_chan, top_chan):
-    """ Modify output channel in-place with the combination of bottom and top channels.
+def blend_channels_multiply(bottom_chan, top_chan):
+    """ Return combination of bottom and top channels.
     
         Math from http://illusions.hu/effectwiki/doku.php?id=multiply_blending
     """
-    output_chan[:,:] = bottom_chan[:,:] * top_chan[:,:]
+    return bottom_chan[:,:] * top_chan[:,:]
 
-def blend_channels_linear_light(output_chan, bottom_chan, top_chan):
-    """ Modify output channel in-place with the combination of bottom and top channels.
+def blend_channels_linear_light(bottom_chan, top_chan):
+    """ Return combination of bottom and top channels.
     
         Math from http://illusions.hu/effectwiki/doku.php?id=linear_light_blending
     """
-    output_chan[:,:] = numpy.clip(bottom_chan[:,:] + 2 * top_chan[:,:] - 1, 0, 1)
+    return numpy.clip(bottom_chan[:,:] + 2 * top_chan[:,:] - 1, 0, 1)
 
-def blend_channels_hard_light(output_chan, bottom_chan, top_chan):
-    """ Modify output channel in-place with the combination of bottom and top channels.
+def blend_channels_hard_light(bottom_chan, top_chan):
+    """ Return combination of bottom and top channels.
     
         Math from http://illusions.hu/effectwiki/doku.php?id=hard_light_blending
     """
     # different pixel subsets for dark and light parts of overlay
     dk, lt = top_chan < .5, top_chan >= .5
     
+    output_chan = numpy.empty(bottom_chan.shape, bottom_chan.dtype)
     output_chan[dk] = 2 * bottom_chan[dk] * top_chan[dk]
     output_chan[lt] = 1 - 2 * (1 - bottom_chan[lt]) * (1 - top_chan[lt])
+    
+    return output_chan
 
 def makeColor(color):
     """ An old name for the make_color function, deprecated for the next version.
