@@ -4,7 +4,7 @@ Takes the first layer from the given mapnik xml file and renders it as UTFGrid
 https://github.com/mapbox/utfgrid-spec/blob/master/1.2/utfgrid.md
 It can then be used for this:
 http://mapbox.github.com/wax/interaction-leaf.html
-Only works with mapnik2 (Where the Grid functionality was introduced)
+Only works with mapnik>=2.0 (Where the Grid functionality was introduced)
 
 Use Sperical Mercator projection and the extension "json"
 
@@ -28,18 +28,17 @@ fields: The fields that should be added to the resulting grid json.
 layer_index: The index of the layer you want from your map xml to be rendered
 wrapper: If not included the json will be output raw, if included the json will be wrapped in "wrapper(JSON)" (for use with wax)
 scale: What to divide the tile pixel size by to get the resulting grid size. Usually this is 4.
+buffer: buffer around the queried features, in px, default 0. Use this to prevent problems on tile boundaries.
 """
 import json
 from TileStache.Core import KnownUnknown
 from TileStache.Geography import getProjectionByName
+from urlparse import urlparse, urljoin
 
 try:
-    import mapnik2 as mapnik
+    import mapnik
 except ImportError:
-    try:
-        import mapnik
-    except ImportError:
-        pass
+    pass
 
 class Provider:
 
@@ -48,7 +47,15 @@ class Provider:
         """
         self.mapnik = None
         self.layer = layer
-        self.mapfile = mapfile
+
+        maphref = urljoin(layer.config.dirpath, mapfile)
+        scheme, h, path, q, p, f = urlparse(maphref)
+        
+        if scheme in ('file', ''):
+            self.mapfile = path
+        else:
+            self.mapfile = maphref
+        
         self.layer_index = layer_index
         self.wrapper = wrapper
         self.scale = scale
@@ -99,7 +106,7 @@ class Provider:
         if extension.lower() != 'json':
             raise KnownUnknown('MapnikGrid only makes .json tiles, not "%s"' % extension)
 
-        return 'application/json', 'JSON'
+        return 'application/json; charset=utf-8', 'JSON'
 
 class SaveableResponse:
     """ Wrapper class for JSON response that makes it behave like a PIL.Image object.
