@@ -8,7 +8,8 @@ Example configuration:
   "cache": {
     "name": "Memcache",
     "servers": ["127.0.0.1:11211"],
-    "revision": 0
+    "revision": 0,
+    "key prefix": "unique-id"
   }
 
 Memcache cache parameters:
@@ -20,6 +21,15 @@ Memcache cache parameters:
   revision
     Optional revision number for mass-expiry of cached tiles
     regardless of lifespan. Defaults to 0.
+
+  key prefix
+    Optional string to prepend to Memcache generated key.
+    Useful when running multiple instances of TileStache
+    that share the same Memcache instance to avoid key
+    collisions. The key prefix will be prepended to the
+    key name. Defaults to "".
+    
+
 """
 from time import time as _time, sleep as _sleep
 
@@ -29,19 +39,20 @@ except ImportError:
     # at least we can build the documentation
     pass
 
-def tile_key(layer, coord, format, rev):
+def tile_key(layer, coord, format, rev, key_prefix):
     """ Return a tile key string.
     """
     name = layer.name()
     tile = '%(zoom)d/%(column)d/%(row)d' % coord.__dict__
-    return str('%(rev)s/%(name)s/%(tile)s.%(format)s' % locals())
+    return str('%(key_prefix)s/%(rev)s/%(name)s/%(tile)s.%(format)s' % locals())
 
 class Cache:
     """
     """
-    def __init__(self, servers=['127.0.0.1:11211'], revision=0):
+    def __init__(self, servers=['127.0.0.1:11211'], revision=0, key_prefix=''):
         self.servers = servers
         self.revision = revision
+        self.key_prefix = key_prefix
 
     def lock(self, layer, coord, format):
         """ Acquire a cache lock for this tile.
@@ -49,7 +60,7 @@ class Cache:
             Returns nothing, but blocks until the lock has been acquired.
         """
         mem = Client(self.servers)
-        key = tile_key(layer, coord, format, self.revision)
+        key = tile_key(layer, coord, format, self.revision, self.key_prefix)
         due = _time() + layer.stale_lock_timeout
         
         try:
@@ -69,7 +80,7 @@ class Cache:
         """ Release a cache lock for this tile.
         """
         mem = Client(self.servers)
-        key = tile_key(layer, coord, format, self.revision)
+        key = tile_key(layer, coord, format, self.revision, self.key_prefix)
         
         mem.delete(key+'-lock')
         mem.disconnect_all()
@@ -78,15 +89,16 @@ class Cache:
         """ Remove a cached tile.
         """
         mem = Client(self.servers)
-        key = tile_key(layer, coord, format, self.revision)
+        key = tile_key(layer, coord, format, self.revision, self.key_prefix)
         
         mem.delete(key)
+        mem.disconnect_all()
         
     def read(self, layer, coord, format):
         """ Read a cached tile.
         """
         mem = Client(self.servers)
-        key = tile_key(layer, coord, format, self.revision)
+        key = tile_key(layer, coord, format, self.revision, self.key_prefix)
         
         value = mem.get(key)
         mem.disconnect_all()
@@ -97,6 +109,7 @@ class Cache:
         """ Save a cached tile.
         """
         mem = Client(self.servers)
-        key = tile_key(layer, coord, format, self.revision)
+        key = tile_key(layer, coord, format, self.revision, self.key_prefix)
         
         mem.set(key, body, layer.cache_lifespan or 0)
+        mem.disconnect_all()
