@@ -34,9 +34,10 @@ via Github to suggest other formats: http://github.com/migurski/TileStache.
 Common parameters:
 
   driver:
-    String used to identify an OGR driver. Currently, only "ESRI Shapefile",
-    "PostgreSQL", and "GeoJSON" are supported as data source drivers, with
-    "postgis" and "shapefile" accepted as synonyms. Not case-sensitive.
+    String used to identify an OGR driver. Currently, "ESRI Shapefile",
+    "PostgreSQL", "MySQL", Oracle, Spatialite and "GeoJSON" are supported as 
+     data source drivers, with "postgis" and "shapefile" accepted as synonyms. 
+     Not case-sensitive.
     
     OGR's complete list of potential formats can be found here:
     http://www.gdal.org/ogr/ogr_formats.html. Feel free to get in touch via
@@ -91,6 +92,11 @@ Common parameters:
     Boolean flag for optionally expanding output with additional whitespace
     for readability. Results in larger but more readable GeoJSON responses.
 
+  id_property:
+    Default is None.
+    Sets the id of the geojson feature to the specified field of the data source.
+    This can be used, for example, to identify a unique key field for the feature.
+
 Example TileStache provider configuration:
 
   "vector-postgis-points":
@@ -155,11 +161,7 @@ try:
 except ImportError:
     from simplejson import JSONEncoder, loads as json_loads
 
-try:
-    from osgeo import ogr, osr
-except ImportError:
-    # At least we'll be able to build the documentation.
-    pass
+from osgeo import ogr, osr
 
 from TileStache.Core import KnownUnknown
 from TileStache.Geography import getProjectionByName
@@ -323,12 +325,17 @@ def _feature_properties(feature, layer_definition, whitelist=None):
         OFTBinary (8), OFTDate (9), OFTTime (10), OFTDateTime (11).
     """
     properties = {}
-    okay_types = ogr.OFTInteger, ogr.OFTReal, ogr.OFTString, ogr.OFTWideString
+    okay_types = ogr.OFTInteger, ogr.OFTReal, ogr.OFTString, ogr.OFTWideString, ogr.OFTDate, ogr.OFTTime, ogr.OFTDateTime
     
     for index in range(layer_definition.GetFieldCount()):
         field_definition = layer_definition.GetFieldDefn(index)
         field_type = field_definition.GetType()
         
+        name = field_definition.GetNameRef()
+        
+        if type(whitelist) in (list, dict) and name not in whitelist:
+            continue
+      
         if field_type not in okay_types:
             try:
                 name = [oft for oft in dir(ogr) if oft.startswith('OFT') and getattr(ogr, oft) == field_type][0]
@@ -336,17 +343,18 @@ def _feature_properties(feature, layer_definition, whitelist=None):
                 raise KnownUnknown("Found an OGR field type I've never even seen: %d" % field_type)
             else:
                 raise KnownUnknown("Found an OGR field type I don't know what to do with: ogr.%s" % name)
-
-        name = field_definition.GetNameRef()
-        
-        if type(whitelist) in (list, dict) and name not in whitelist:
-            continue
-        
+       
         property = type(whitelist) is dict and whitelist[name] or name
         properties[property] = feature.GetField(name)
     
     return properties
 
+def _append_with_delim(s, delim, data, key):
+    if key in data:
+        return s + delim + str(data[key])
+    else:
+        return s
+	
 def _open_layer(driver_name, parameters, dirpath):
     """ Open a layer, return it and its datasource.
     
@@ -357,7 +365,7 @@ def _open_layer(driver_name, parameters, dirpath):
     #
     okay_drivers = {'postgis': 'PostgreSQL', 'esri shapefile': 'ESRI Shapefile',
                     'postgresql': 'PostgreSQL', 'shapefile': 'ESRI Shapefile',
-                    'geojson': 'GeoJSON'}
+                    'geojson': 'GeoJSON', 'spatialite': 'SQLite', 'oracle': 'OCI', 'mysql': 'MySQL'}
     
     if driver_name.lower() not in okay_drivers:
         raise KnownUnknown('Got a driver type Vector doesn\'t understand: "%s". Need one of %s.' % (driver_name, ', '.join(okay_drivers.keys())))
@@ -379,8 +387,37 @@ def _open_layer(driver_name, parameters, dirpath):
                 conn_parts.append("%s='%s'" % (part, parameters[part]))
         
         source_name = 'PG:' + ' '.join(conn_parts)
+
+    elif driver_name == 'MySQL':
+        if 'dbname' not in parameters:
+            raise KnownUnknown('Need a "dbname" parameter for MySQL')
+        if 'table' not in parameters:
+            raise KnownUnknown('Need a "table" parameter for MySQL')
+
+        conn_parts = []
         
-    elif driver_name in ('ESRI Shapefile', 'GeoJSON'):
+        for part in ('host', 'port', 'user', 'password'):
+            if part in parameters:
+                conn_parts.append("%s=%s" % (part, parameters[part]))
+        
+        source_name = 'MySql:' + parameters["dbname"] + "," + ','.join(conn_parts) + ",tables=" + parameters['table']
+
+    elif driver_name == 'OCI':
+        if 'host' not in parameters:
+            raise KnownUnknown('Need a "host" parameter for oracle')
+        if 'table' not in parameters:
+            raise KnownUnknown('Need a "table" parameter for oracle')
+        source_name = 'OCI:'
+        source_name = _append_with_delim(source_name, '', parameters, 'user')
+        source_name = _append_with_delim(source_name, '/', parameters, 'password')
+        if 'user' in parameters:
+	        source_name = source_name + '@'
+        source_name = source_name + parameters['host']
+        source_name = _append_with_delim(source_name, ':', parameters, 'port')
+        source_name = _append_with_delim(source_name, '/', parameters, 'dbname')
+        source_name = source_name + ":" + parameters['table']
+        
+    elif driver_name in ('ESRI Shapefile', 'GeoJSON', 'SQLite'):
         if 'file' not in parameters:
             raise KnownUnknown('Need at least a "file" parameter for a shapefile')
     
@@ -400,18 +437,19 @@ def _open_layer(driver_name, parameters, dirpath):
     #
     # Set up the layer
     #
-    if driver_name == 'PostgreSQL':
+    if driver_name == 'PostgreSQL' or driver_name == 'OCI' or driver_name == 'MySQL':
         if 'query' in parameters:
             layer = datasource.ExecuteSQL(str(parameters['query']))
         elif 'table' in parameters:
             layer = datasource.GetLayerByName(str(parameters['table']))
         else:
-            raise KnownUnknown('Need at least a "query" or "table" parameter for postgis')
-
+            raise KnownUnknown('Need at least a "query" or "table" parameter for postgis or oracle')
+    elif driver_name == 'SQLite':
+        layer = datasource.GetLayerByName(str(parameters['layer']))
     else:
         layer = datasource.GetLayer(0)
 
-    if layer.GetSpatialRef() is None: 
+    if layer.GetSpatialRef() is None and driver_name != 'SQLite': 
         raise KnownUnknown('Couldn\'t get a layer from data source %s' % source_name)
 
     #
@@ -422,7 +460,7 @@ def _open_layer(driver_name, parameters, dirpath):
     #
     return layer, datasource
 
-def _get_features(coord, properties, projection, layer, clipped, projected, spacing):
+def _get_features(coord, properties, projection, layer, clipped, projected, spacing, id_property):
     """ Return a list of features in an OGR layer with properties in GeoJSON form.
     
         Optionally clip features to coordinate bounding box, and optionally
@@ -443,6 +481,8 @@ def _get_features(coord, properties, projection, layer, clipped, projected, spac
     #
     definition = layer.GetLayerDefn()
     layer_sref = layer.GetSpatialRef()
+    if layer_sref == None:
+        layer_sref = _sref_4326()
     
     #
     # Spatially filter the layer
@@ -472,6 +512,7 @@ def _get_features(coord, properties, projection, layer, clipped, projected, spac
         if geometry is None:
             # may indicate a TopologyException
             continue
+
         
         # mask out subsequent features if spacing is defined
         if mask and buffer:
@@ -484,8 +525,11 @@ def _get_features(coord, properties, projection, layer, clipped, projected, spac
 
         geom = json_loads(geometry.ExportToJson())
         prop = _feature_properties(feature, definition, properties)
-        
-        features.append({'type': 'Feature', 'properties': prop, 'geometry': geom})
+
+        geojson_feature = {'type': 'Feature', 'properties': prop, 'geometry': geom}
+        if id_property != None and id_property in prop:
+           geojson_feature['id'] = prop[id_property]
+        features.append(geojson_feature)
     
     return features
 
@@ -495,7 +539,7 @@ class Provider:
         See module documentation for explanation of constructor arguments.
     """
     
-    def __init__(self, layer, driver, parameters, clipped, verbose, projected, spacing, properties, precision):
+    def __init__(self, layer, driver, parameters, clipped, verbose, projected, spacing, properties, precision, id_property):
         self.layer      = layer
         self.driver     = driver
         self.clipped    = clipped
@@ -505,12 +549,39 @@ class Provider:
         self.parameters = parameters
         self.properties = properties
         self.precision  = precision
+        self.id_property = id_property
 
+    @staticmethod
+    def prepareKeywordArgs(config_dict):
+        """ Convert configured parameters to keyword args for __init__().
+        """
+        kwargs = dict()
+        
+        kwargs['driver'] = config_dict['driver']
+        kwargs['parameters'] = config_dict['parameters']
+        kwargs['id_property'] = config_dict.get('id_property', None)
+        kwargs['properties'] = config_dict.get('properties', None)
+        kwargs['projected'] = bool(config_dict.get('projected', False))
+        kwargs['verbose'] = bool(config_dict.get('verbose', False))
+        kwargs['precision'] = int(config_dict.get('precision', 6))
+        
+        if 'spacing' in config_dict:
+            kwargs['spacing'] = float(config_dict.get('spacing', 0.0))
+        else:
+            kwargs['spacing'] = None
+        
+        if config_dict.get('clipped', None) == 'padded':
+            kwargs['clipped'] = 'padded'
+        else:
+            kwargs['clipped'] = bool(config_dict.get('clipped', True))
+        
+        return kwargs
+    
     def renderTile(self, width, height, srs, coord):
         """ Render a single tile, return a VectorResponse instance.
         """
         layer, ds = _open_layer(self.driver, self.parameters, self.layer.config.dirpath)
-        features = _get_features(coord, self.properties, self.layer.projection, layer, self.clipped, self.projected, self.spacing)
+        features = _get_features(coord, self.properties, self.layer.projection, layer, self.clipped, self.projected, self.spacing, self.id_property)
         response = {'type': 'FeatureCollection', 'features': features}
         
         if self.projected:
@@ -531,10 +602,10 @@ class Provider:
             This only accepts "geojson" for the time being.
         """
         if extension.lower() == 'geojson':
-            return 'text/json', 'GeoJSON'
+            return 'application/json', 'GeoJSON'
     
         elif extension.lower() == 'arcjson':
-            return 'text/json', 'ArcJSON'
+            return 'application/json', 'ArcJSON'
             
         elif extension.lower() == 'geobson':
             return 'application/x-bson', 'GeoBSON'
