@@ -10,12 +10,17 @@ See `tilestache-seed.py --help` for more information.
 """
 
 from sys import stderr, path
+from os.path import realpath, dirname
 from optparse import OptionParser
+from urlparse import urlparse
+from urllib import urlopen
 
 try:
     from json import dump as json_dump
+    from json import load as json_load
 except ImportError:
     from simplejson import dump as json_dump
+    from simplejson import load as json_load
 
 #
 # Most imports can be found below, after the --include-path option is known.
@@ -38,7 +43,7 @@ Protip: extract tiles from an MBTiles tileset to a directory like this:
 
 Configuration, bbox, and layer options are required; see `%prog --help` for info.""")
 
-defaults = dict(extension='png', padding=0, verbose=True, enable_retries=False, bbox=(37.777, -122.352, 37.839, -122.226))
+defaults = dict(padding=0, verbose=True, enable_retries=False, bbox=(37.777, -122.352, 37.839, -122.226))
 
 parser.set_defaults(**defaults)
 
@@ -57,7 +62,7 @@ parser.add_option('-p', '--padding', dest='padding',
                   type='int')
 
 parser.add_option('-e', '--extension', dest='extension',
-                  help='Optional file type for rendered tiles. Default value is %s.' % repr(defaults['extension']))
+                  help='Optional file type for rendered tiles. Default value is "png" for most image layers and some variety of JSON for Vector or Mapnik Grid providers.')
 
 parser.add_option('-f', '--progress-file', dest='progressfile',
                   help="Optional JSON progress file that gets written on each iteration, so you don't have to pay close attention.")
@@ -74,6 +79,10 @@ parser.add_option('-d', '--output-directory', dest='outputdirectory',
 parser.add_option('--to-mbtiles', dest='mbtiles_output',
                   help='Optional output file for tiles, will be created as an MBTiles 1.1 tileset. See http://mbtiles.org for more information.')
 
+parser.add_option('--to-s3', dest='s3_output',
+                  help='Optional output bucket for tiles, will be populated with tiles in a standard Z/X/Y layout. Three required arguments: AWS access-key, secret, and bucket name.',
+                  nargs=3)
+
 parser.add_option('--from-mbtiles', dest='mbtiles_input',
                   help='Optional input file for tiles, will be read as an MBTiles 1.1 tileset. See http://mbtiles.org for more information. Overrides --extension, --bbox and --padding (this may change).')
 
@@ -90,6 +99,9 @@ parser.add_option('--enable-retries', dest='enable_retries',
 parser.add_option('-x', '--ignore-cached', action='store_true', dest='ignore_cached',
                   help='Re-render every tile, whether it is in the cache already or not.')
 
+parser.add_option('--jsonp-callback', dest='callback',
+                  help='Add a JSONP callback for tiles with a json mime-type, causing "*.js" tiles to be written to the cache wrapped in the callback function. Ignored for non-JSON tiles.')
+
 def generateCoordinates(ul, lr, zooms, padding):
     """ Generate a stream of (offset, count, coordinate) tuples for seeding.
     
@@ -115,8 +127,8 @@ def generateCoordinates(ul, lr, zooms, padding):
         ul_ = ul.zoomTo(zoom).container().left(padding).up(padding)
         lr_ = lr.zoomTo(zoom).container().right(padding).down(padding)
 
-        for row in range(int(ul_.row), int(lr_.row + 1)):
-            for column in range(int(ul_.column), int(lr_.column + 1)):
+        for row in xrange(int(ul_.row), int(lr_.row + 1)):
+            for column in xrange(int(ul_.column), int(lr_.column + 1)):
                 coord = Coordinate(row, column, zoom)
                 
                 yield (offset, count, coord)
@@ -148,6 +160,23 @@ def tilesetCoordinates(filename):
     for (offset, coord) in enumerate(coords):
         yield (offset, count, coord)
 
+def parseConfigfile(configpath):
+    """ Parse a configuration file and return a raw dictionary and dirpath.
+    
+        Return value can be passed to TileStache.Config.buildConfiguration().
+    """
+    config_dict = json_load(urlopen(configpath))
+    
+    scheme, host, path, p, q, f = urlparse(configpath)
+    
+    if scheme == '':
+        scheme = 'file'
+        path = realpath(path)
+    
+    dirpath = '%s://%s%s' % (scheme, host, dirname(path).rstrip('/') + '/')
+    
+    return config_dict, dirpath
+
 if __name__ == '__main__':
     options, zooms = parser.parse_args()
 
@@ -155,9 +184,9 @@ if __name__ == '__main__':
         for p in options.include_paths.split(':'):
             path.insert(0, p)
 
-    from TileStache import parseConfigfile, getTile
+    from TileStache import getTile, Config
     from TileStache.Core import KnownUnknown
-    from TileStache.Caches import Disk, Multi, Test
+    from TileStache.Config import buildConfiguration
     from TileStache import MBTiles
     import TileStache
     
@@ -171,13 +200,11 @@ if __name__ == '__main__':
         has_fake_source = bool(options.mbtiles_input)
         
         if has_fake_destination and has_fake_source:
-            config = TileStache.Config.Configuration(Test(), '.')
-
-            metatile = TileStache.Core.Metatile()
-            projection = TileStache.Geography.SphericalMercator()
-            layer = TileStache.Core.Layer(config, projection, metatile)
+            config_dict, config_dirpath = dict(layers={}), '' # parseConfigfile(options.config)
+            layer_dict = dict()
             
-            config.layers[options.layer or 'tiles-layer'] = layer
+            config_dict['cache'] = dict(name='test')
+            config_dict['layers'][options.layer or 'tiles-layer'] = layer_dict
         
         elif options.config is None:
             raise KnownUnknown('Missing required configuration (--config) parameter.')
@@ -186,34 +213,67 @@ if __name__ == '__main__':
             raise KnownUnknown('Missing required layer (--layer) parameter.')
     
         else:
-            config = parseConfigfile(options.config)
+            config_dict, config_dirpath = parseConfigfile(options.config)
             
-            if options.layer not in config.layers:
-                raise KnownUnknown('"%s" is not a layer I know about. Here are some that I do know about: %s.' % (options.layer, ', '.join(sorted(config.layers.keys()))))
+            if options.layer not in config_dict['layers']:
+                raise KnownUnknown('"%s" is not a layer I know about. Here are some that I do know about: %s.' % (options.layer, ', '.join(sorted(config_dict['layers'].keys()))))
             
-            layer = config.layers[options.layer]
-            layer.write_cache = True # Override to make seeding guaranteed useful.
+            layer_dict = config_dict['layers'][options.layer]
+            layer_dict['write_cache'] = True # Override to make seeding guaranteed useful.
         
         # override parts of the config and layer if needed
         
         extension = options.extension
-        
-        if options.outputdirectory and options.mbtiles_output:
-            cache1 = Disk(options.outputdirectory, dirs='portable', gzip=[])
-            cache2 = MBTiles.Cache(options.mbtiles_output, extension, options.layer)
-            config.cache = Multi([cache1, cache2])
 
-        elif options.outputdirectory:
-            config.cache = Disk(options.outputdirectory, dirs='portable', gzip=[])
-
-        elif options.mbtiles_output:
-            config.cache = MBTiles.Cache(options.mbtiles_output, extension, options.layer)
-        
         if options.mbtiles_input:
-            layer.provider = MBTiles.Provider(layer, options.mbtiles_input)
+            layer_dict['provider'] = dict(name='mbtiles', tileset=options.mbtiles_input)
             n, t, v, d, format, b = MBTiles.tileset_info(options.mbtiles_input)
             extension = format or extension
         
+        # determine or guess an appropriate tile extension
+        
+        if extension is None:
+            provider_name = layer_dict['provider'].get('name', '').lower()
+            
+            if provider_name == 'mapnik grid':
+                extension = 'json'
+            elif provider_name == 'vector':
+                extension = 'geojson'
+            else:
+                extension = 'png'
+        
+        # override parts of the config and layer if needed
+        
+        tiers = []
+        
+        if options.mbtiles_output:
+            tiers.append({'class': 'TileStache.MBTiles:Cache',
+                          'kwargs': dict(filename=options.mbtiles_output,
+                                         format=extension,
+                                         name=options.layer)})
+        
+        if options.outputdirectory:
+            tiers.append(dict(name='disk', path=options.outputdirectory,
+                              dirs='portable', gzip=[]))
+
+        if options.s3_output:
+            access, secret, bucket = options.s3_output
+            tiers.append(dict(name='S3', bucket=bucket,
+                              access=access, secret=secret))
+        
+        if len(tiers) > 1:
+            config_dict['cache'] = dict(name='multi', tiers=tiers)
+        elif len(tiers) == 1:
+            config_dict['cache'] = tiers[0]
+        else:
+            # Leave config_dict['cache'] as-is
+            pass
+        
+        # create a real config object
+        
+        config = buildConfiguration(config_dict, config_dirpath)
+        layer = config.layers[options.layer or 'tiles-layer']
+        
         # do the actual work
         
         lat1, lon1, lat2, lon2 = options.bbox
@@ -249,8 +309,6 @@ if __name__ == '__main__':
     else:
         coordinates = generateCoordinates(ul, lr, zooms, padding)
     
-    coordinates = list(coordinates)
-    
     for (offset, count, coord) in coordinates:
         path = '%s/%d/%d/%d.%s' % (layer.name(), coord.zoom, coord.column, coord.row, extension)
 
@@ -271,6 +329,17 @@ if __name__ == '__main__':
     
             try:
                 mimetype, content = getTile(layer, coord, extension, options.ignore_cached)
+                
+                if mimetype and 'json' in mimetype and options.callback:
+                    js_path = '%s/%d/%d/%d.js' % (layer.name(), coord.zoom, coord.column, coord.row)
+                    js_body = '%s(%s);' % (options.callback, content)
+                    js_size = len(js_body) / 1024
+                    
+                    layer.config.cache.save(js_body, layer, coord, 'JS')
+                    print >> stderr, '%s (%dKB)' % (js_path, js_size),
+            
+                elif options.callback:
+                    print >> stderr, '(callback ignored)',
             
             except:
                 #
